export function followRedone(store: StructStore, id: ID): {
    item: Item;
    diff: number;
};
export function keepItem(item: Item | null, keep: boolean): void;
export function splitItem(transaction: Transaction, leftItem: Item, diff: number): Item;
export function redoItem(transaction: Transaction, item: Item, redoitems: Set<Item>, itemsToDelete: DeleteSet, ignoreRemoteMapChanges: boolean): Item | null;
/**
 * Abstract class that represents any content.
 */
export class Item extends AbstractStruct {
    /**
     * @param {ID} id
     * @param {Item | null} left
     * @param {ID | null} origin
     * @param {Item | null} right
     * @param {ID | null} rightOrigin
     * @param {AbstractType<any>|ID|null} parent Is a type if integrated, is null if it is possible to copy parent from left or right, is ID before integration to search for it.
     * @param {string | null} parentSub
     * @param {AbstractContent} content
     * @param {Array<Signature> | null | undefined} signature
     * @param {string} source
     * @param {IntegrateSignature | null | undefined} integrateSignature
     * @param {Array<DeleteSignature> | null | undefined} deleteSignature
     */
    constructor(id: ID, left: Item | null, origin: ID | null, right: Item | null, rightOrigin: ID | null, parent: AbstractType<any> | ID | null, parentSub: string | null, content: AbstractContent, signature?: Array<Signature> | null | undefined, source?: string, integrateSignature?: IntegrateSignature | null | undefined, deleteSignature?: Array<DeleteSignature> | null | undefined);
    /**
     * The item that was originally to the left of this item.
     * @type {ID | null}
     */
    origin: ID | null;
    /**
     * The item that is currently to the left of this item.
     * @type {Item | null}
     */
    left: Item | null;
    /**
     * The item that is currently to the right of this item.
     * @type {Item | null}
     */
    right: Item | null;
    /**
     * The item that was originally to the right of this item.
     * @type {ID | null}
     */
    rightOrigin: ID | null;
    /**
     * @type {AbstractType<any>|ID|null}
     */
    parent: AbstractType<any> | ID | null;
    /**
     * If the parent refers to this item with some kind of key (e.g. YMap, the
     * key is specified here. The key is then used to refer to the list in which
     * to insert this item. If `parentSub = null` type._start is the list in
     * which to insert to. Otherwise it is `parent._map`.
     * @type {String | null}
     */
    parentSub: string | null;
    /**
     * If this type's effect is redone this type refers to the type that undid
     * this operation.
     * @type {ID | null}
     */
    redone: ID | null;
    /**
     * @type {AbstractContent}
     */
    content: AbstractContent;
    /**
     * bit1: keep
     * bit2: countable
     * bit3: deleted
     * bit4: mark - mark node as fast-search-marker
     * @type {number} byte
     */
    info: number;
    hidden: boolean;
    signature: Signature[] | null;
    integrateSignature: IntegrateSignature | null;
    /**
     * @type {Array<DeleteSignature> | null}
     */
    deleteSignature: Array<DeleteSignature> | null;
    source: string;
    /**
     * This is used to mark the item as an indexed fast-search marker
     *
     * @type {boolean}
     */
    set marker(arg: boolean);
    get marker(): boolean;
    set keep(arg: boolean);
    /**
     * If true, do not garbage collect this Item.
     */
    get keep(): boolean;
    get countable(): boolean;
    set deleted(arg: boolean);
    /**
     * Whether this item was deleted or not.
     * @type {Boolean}
     */
    get deleted(): boolean;
    markDeleted(): void;
    /**
     * Return the creator clientID of the missing op or define missing items and return null.
     *
     * @param {Transaction} transaction
     * @param {StructStore} store
     * @return {null | number}
     */
    getMissing(transaction: Transaction, store: StructStore): null | number;
    /**
     *
     * @returns { string | Array<number> | undefined }
     */
    getParentSignal(): string | Array<number> | undefined;
    /**
     * @private
     * @param {*} transaction
     * @param {*} offset
     */
    private check;
    /**
     * @private
     * @param {*} transaction
     * @param {*} offset
     * @param {Function} successNext
     * @param {Function} failNext
     */
    private checkIntegrateSignature;
    /**
     * @public
     * @param {Transaction} transaction
     * @param {number} offset
     * @param {Function} successNext
     * @param {Function} failNext
     *
     */
    public checkAtomicSignature(transaction: Transaction, offset: number, successNext?: Function, failNext?: Function): void;
    /**
     * @private
     * @param {Transaction} transaction
     * @param {number} offset
     * @param {Function} successNext
     */
    private checkSignatureIterate;
    /**
     * 签名校验过程
     * @param {Transaction} transaction
     * @param {number} offset
     * @param {Function} successNext
     */
    checkSignature(transaction: Transaction, offset: number, successNext?: Function): void;
    /**
     * Returns the next non-deleted item
     */
    get next(): Item | null;
    /**
     * Returns the previous non-deleted item
     */
    get prev(): Item | null;
    /**
     * Computes the last content address of this Item.
     */
    get lastId(): ID;
    /**
     * Try to merge two items
     *
     * @param {Item} right
     * @return {boolean}
     */
    mergeWith(right: Item): boolean;
    /**
     * @private
     * @param {Transaction} transaction
     * @param {number} index
     * @returns
     */
    private deleteByIndex;
    /**
     * Mark this Item as deleted.
     *
     * @param {Transaction} transaction
     * @param {Array<DeleteSignature> | null} signature
     */
    delete(transaction: Transaction, signature?: Array<DeleteSignature> | null): void;
    /**
     * Mark this Item as deleted.
     * @private
     * @param {Transaction} transaction
     * @param {Array<DeleteSignature> | null} signature
     * @param {Function} successNext
     * @param {Function} failNext
     */
    private applyPreDeleteProcess;
    /**
     * 执行删除逻辑
     * @param {Transaction} transaction
     */
    applyDeleteProcess(transaction: Transaction): void;
    /**
     * 检查删除签名的合法性
     * @param {Transaction} transaction
     * @param {Function} successNext
     * @param {Function} failNext
     */
    checkDeleteSignature(transaction: Transaction, successNext?: Function, failNext?: Function): false | undefined;
    /**
     * Mark this Item as hidden or not.
     * hidden操作不会修改长度，只是隐藏item
     *
     * @param {Transaction} transaction
     */
    hide(transaction: Transaction, hidden?: boolean, offset?: number): void;
    /**
     * @private
     * @param {*} transaction
     * @param {*} hidden
     */
    private applyHideProcess;
    /**
     * @param {StructStore} store
     * @param {boolean} parentGCd
     */
    gc(store: StructStore, parentGCd: boolean): void;
    /**
     * signal a unique item, except origin/left and so on not immutable attr
     * item可能是多个item进行压缩的，所以需要找到确切的位置
     * @param {UpdateEncoderV1} encoder
     * @param {number} offset
     * @param {boolean} needContent
     */
    signal(encoder: UpdateEncoderV1, offset: number, needContent?: boolean): void;
    /**
     * Transform the properties of this type to binary and write it to an
     * BinaryEncoder.
     *
     * This is called when this Item is sent to a remote peer.
     *
     * @param {UpdateEncoderV1 | UpdateEncoderV2} encoder The encoder to write data to.
     * @param {number} offset
     */
    write(encoder: UpdateEncoderV1 | UpdateEncoderV2, offset: number): void;
    /**
     * @private
     * @param {UpdateEncoderV1 | UpdateEncoderV2} encoder The encoder to write data to.
     * @param {number} offset
     */
    private writeSignature;
    /**
     * @private
     * @param {UpdateEncoderV1 | UpdateEncoderV2} encoder The encoder to write data to.
     */
    private writeIntegrateSignature;
}
export function readItemContent(decoder: UpdateDecoderV1 | UpdateDecoderV2, info: number): AbstractContent;
/**
 * A lookup map for reading Item content.
 *
 * @type {Array<function(UpdateDecoderV1 | UpdateDecoderV2):AbstractContent>}
 */
export const contentRefs: Array<(arg0: UpdateDecoderV1 | UpdateDecoderV2) => AbstractContent>;
/**
 * Do not implement this class!
 */
export class AbstractContent {
    /**
     * @return {number}
     */
    getLength(): number;
    /**
     * @return {Array<any>}
     */
    getContent(): Array<any>;
    /**
     * Should return false if this Item is some kind of meta information
     * (e.g. format information).
     *
     * * Whether this Item should be addressable via `yarray.get(i)`
     * * Whether this Item should be counted when computing yarray.length
     *
     * @return {boolean}
     */
    isCountable(): boolean;
    /**
     * @return {AbstractContent}
     */
    copy(): AbstractContent;
    /**
     * @param {number} offset
     * @return {AbstractContent}
     */
    splice(offset: number): AbstractContent;
    /**
     * @param {AbstractContent} right
     * @return {boolean}
     */
    mergeWith(right: AbstractContent): boolean;
    /**
     * @param {Transaction} transaction
     * @param {Item} item
     */
    integrate(transaction: Transaction, item: Item): void;
    /**
     * @param {Transaction} transaction
     */
    delete(transaction: Transaction): void;
    /**
     * @param {StructStore} store
     */
    gc(store: StructStore): void;
    /**
     * @param {UpdateEncoderV1 | UpdateEncoderV2} encoder
     * @param {number} offset
     */
    write(encoder: UpdateEncoderV1 | UpdateEncoderV2, offset: number): void;
    /**
     * @param {UpdateEncoderV1 | UpdateEncoderV2} encoder
     * @param {number} offset
     */
    signal(encoder: UpdateEncoderV1 | UpdateEncoderV2, offset: number): void;
    /**
     * @return {number}
     */
    getRef(): number;
}
import { StructStore } from "../utils/StructStore.js";
import { ID } from "../utils/ID.js";
import { Transaction } from "../utils/Transaction.js";
import { DeleteSet } from "../utils/DeleteSet.js";
import { AbstractStruct } from "./AbstractStruct.js";
import { AbstractType } from "../types/AbstractType.js";
import { Signature } from "../utils/signature/signature.js";
import { IntegrateSignature } from "../utils/signature/signature.js";
import { DeleteSignature } from "../utils/signature/signature.js";
import { UpdateEncoderV1 } from "../utils/UpdateEncoder.js";
import { UpdateEncoderV2 } from "../utils/UpdateEncoder.js";
import { UpdateDecoderV1 } from "../utils/UpdateDecoder.js";
import { UpdateDecoderV2 } from "../utils/UpdateDecoder.js";
